var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useEffect, useState } from "react";
import get from "lodash/get";
import set from "lodash/set";
import produce from "immer";
import { ProviderConsumer as FluentUIThemeConsumer, Box, } from "@fluentui/react-northstar";
import { Widget, } from "./DashboardWidget";
import { DashboardTheme } from "./DashboardTheme";
import { Sidebar } from "./Sidebar";
import { Toolbar } from "../Toolbar/Toolbar";
var emptyPrefs = { widgetSettings: {} };
/*
 * This method returns the same stored preferences for any Dashboard.
 */
var getStoredPrefs = function (cacheKey) {
    return cacheKey
        ? (function () {
            var storedPrefs = window.localStorage.getItem(cacheKey);
            return storedPrefs ? JSON.parse(storedPrefs) : false;
        })() || emptyPrefs
        : emptyPrefs;
};
/**
 * @public
 */
export function Dashboard(_a) {
    var widgets = _a.widgets, preferences = _a.preferences, cacheKey = _a.cacheKey, onInteraction = _a.onInteraction, blockOnly = _a.blockOnly;
    var _b = useState(false), sidebarOpen = _b[0], setSidebarOpen = _b[1];
    var closeSidebar = function () { return setSidebarOpen(false); };
    var localStorageKey = cacheKey
        ? "@fluentui/react-teams__".concat(cacheKey)
        : undefined;
    var initializePreferencesState = function () {
        return produce(preferences || getStoredPrefs(localStorageKey), function (draft) {
            widgets.reduce(function (draft, _a) {
                var id = _a.id;
                return set(draft, "widgetSettings.".concat(id, ".display"), get(draft, "widgetSettings.".concat(id, ".display"), true));
            }, draft);
        });
    };
    var _c = useState(initializePreferencesState), preferencesState = _c[0], setPreferencesState = _c[1];
    useEffect(function () {
        localStorageKey &&
            window.localStorage.setItem(localStorageKey, JSON.stringify(preferencesState));
    }, [preferencesState]);
    var updatePreferences = function (nextPreferences) {
        setPreferencesState(nextPreferences);
        onInteraction &&
            onInteraction({
                event: "update",
                target: "preferences",
                preferences: nextPreferences,
            });
    };
    var hideWidget = function (widgetId) {
        updatePreferences(produce(preferencesState, function (draft) {
            set(draft, "widgetSettings.".concat(widgetId, ".display"), false);
        }));
    };
    return (React.createElement(FluentUIThemeConsumer, { render: function (globalTheme) {
            var t = globalTheme.siteVariables.t;
            return (React.createElement(DashboardTheme, { globalTheme: globalTheme },
                !blockOnly && (React.createElement(React.Fragment, null,
                    React.createElement(Toolbar, __assign({}, {
                        actionGroups: {
                            h1: {
                                edit: { title: t["edit dashboard"], icon: "Edit" },
                            },
                        },
                        filters: [],
                        find: false,
                    }, { onInteraction: function (_a) {
                            var action = _a.action;
                            switch (action) {
                                case "edit":
                                    setSidebarOpen(true);
                                    break;
                            }
                        } })),
                    React.createElement(Box, { styles: { height: "1.25rem" }, role: "presentation" }),
                    React.createElement(Sidebar, __assign({ open: sidebarOpen, onClose: closeSidebar }, { t: t, widgets: widgets, preferencesState: preferencesState, updatePreferences: updatePreferences })))),
                React.createElement(Box, { styles: {
                        display: "grid",
                        gridGap: ".5rem",
                        gridTemplate: "repeat(auto-fill, 25rem) / repeat(auto-fill, minmax(18.75rem, 1fr))",
                        gridAutoFlow: "dense",
                        gridAutoRows: "25rem",
                        padding: "0 1rem 1.25rem",
                        minWidth: "20rem",
                        "@media (max-width: 986px)": {
                            gridTemplate: "repeat(auto-fill, 25rem) / repeat(auto-fill, minmax(15.75rem, 1fr))",
                        },
                    } }, widgets &&
                    widgets.map(function (_a, key) {
                        var id = _a.id, title = _a.title, desc = _a.desc, widgetActionGroup = _a.widgetActionGroup, widgetFilterGroup = _a.widgetFilterGroup, initialFilter = _a.initialFilter, size = _a.size, body = _a.body, bodyByFilter = _a.bodyByFilter, link = _a.link;
                        return (blockOnly ||
                            get(preferencesState, "widgetSettings.".concat(id, ".display"), true)) && (React.createElement(Widget, __assign({}, __assign(__assign({ key: key, size: size, title: title, desc: desc, id: id, widgetActionGroup: widgetActionGroup, widgetFilterGroup: widgetFilterGroup, initialFilter: initialFilter, onInteraction: onInteraction }, (!blockOnly && { hideWidget: hideWidget })), { body: body, bodyByFilter: bodyByFilter, link: link }))));
                    }))));
        } }));
}
//# sourceMappingURL=Dashboard.js.map